// lzw_main.h: test for the lzw::compression class.
//
//////////////////////////////////////////////////////////////////////

#include <string>
#include <iostream>
#include <sstream>

#include "lzw/lzw_compression.h"

#include "utils/string_utils.h"

int main (int narg, char** argvp)
{
    const char*     input("tomtomtom");
    size_t          length(strlen(input));
    size_t          written(0);
    char            compressed[1024];
    char            decompressed[1204];

    std::cout << "separate compression and decompression: " << std::endl;

    std::cout << "input:        " << to_binary(input) << std::endl;

    {
        lzw::compression    compression;

        written = compression.compress(input, length, compressed, 1024);
    }

    std::cout << "compressed:   " << to_binary(compressed) << std::endl;

    {
        lzw::compression    compression;

        compression.decompress(compressed, written, decompressed, 1024);
    }

    std::cout << "decompressed: " << to_binary(decompressed) << std::endl;
    std::cout << std::endl;

    std::cout << "compressing a large chunk with many repetitions:" << std::endl;
    {
        size_t  length(64*1024*1024);
//        size_t  length(144);
        char*   source(new char[length+1]);
        char*   compre(new char[length+1]);
        char*   decomp(new char[length+1]);
        size_t  compre_length;
        size_t  decomp_length;

        // create some data
        for (size_t i(0); i < length; ++i)
        {
            source[i] = mix(i&0x3f);
        }

        {
            lzw::compression    compression;

            compre_length = compression.compress(source, length, compre, length);

            std::cout << length << " bytes compressed to " << compre_length << " (" << ((100.0*(length-compre_length))/length) << "%)." << std::endl;
        }

        {
            lzw::compression    decompression;

            decomp_length = decompression.decompress(compre, compre_length, decomp, length+1);
        }

        std::cout << compre_length << " bytes decompressed to " << decomp_length << '.' << std::endl;

        if (length == decomp_length)
        {
            for (size_t i(0); i < length; ++i)
            {
                if (decomp[i] != source[i])
                {
                    std::cout << "error after " << i << " bytes" << std::endl;
                    std::cout << "source                           result" << std::endl;

                    for (size_t j(0); j < length; j+=16)
                    {
                        for (size_t k(0); k < 16; ++k)
                        {
                            std::cout << to_hex(source[j+k]);
                        }

                        std::cout << ' ';

                        for (size_t k(0); k < 16; ++k)
                        {
                            std::cout << to_hex(decomp[j+k]);
                        }

                        std::cout << ' ' << to_hex(j) << std::endl;
                    }

                    break;
                }
            }
        }

        delete source;
        delete compre;
        delete decomp;
    }

    return 0;
}
